<?php
/**
 * @file
 * Administration pages for the 'Subscribe to feed' module.
 */

/**
 * Form builder; Settings page for the 'Subscribe to feed' module.
 *
 * @ingroup forms
 */
function subscribe_to_feed_admin_settings() {
  // Page Feed icons
  $form['page_feeds_icons'] = array(
    '#type' => 'fieldset',
    '#title' => t('Page feed icons'),
    '#description' => t('Feed icons are usually displayed at the bottom of the page as an RSS icon with text.'),
  );

  $form['page_feeds_icons']['subscribe_to_feed_page_style'] = array(
    '#type' => 'select',
    '#title' => t('Choice a style'),
    '#default_value' => variable_get('subscribe_to_feed_page_style', SUBSCRIBE_TO_FEED_STYLE_IMAGE_AND_TEXT),
    '#options' => array(
      SUBSCRIBE_TO_FEED_STYLE_DEFAULT => t('Default'),
      SUBSCRIBE_TO_FEED_STYLE_HIDDEN => t('Hidden'),
      SUBSCRIBE_TO_FEED_STYLE_TEXT => t('Text'),
      SUBSCRIBE_TO_FEED_STYLE_IMAGE => t('Image'),
      SUBSCRIBE_TO_FEED_STYLE_IMAGE_AND_TEXT => t('Image and Text')
    ),
  );

  // Views Feed icons
  $form['views_feeds_icons'] = array(
    '#type' => 'fieldset',
    '#title' => t('Views feed icons'),
    '#description' => t('Feed icons are usually displayed at the bottom of a view as an RSS icon with text.'),
  );

  $form['views_feeds_icons']['subscribe_to_feed_view_style'] = array(
    '#type' => 'select',
    '#title' => t('Choice a style'),
    '#default_value' => variable_get('subscribe_to_feed_view_style', SUBSCRIBE_TO_FEED_STYLE_HIDDEN),
    '#options' => array(
      SUBSCRIBE_TO_FEED_STYLE_DEFAULT => t('Default'),
      SUBSCRIBE_TO_FEED_STYLE_HIDDEN => t('Hidden'),
      SUBSCRIBE_TO_FEED_STYLE_TEXT => t('Text'),
      SUBSCRIBE_TO_FEED_STYLE_IMAGE => t('Image'),
      SUBSCRIBE_TO_FEED_STYLE_IMAGE_AND_TEXT => t('Image and Text')
    ),
  );

  // RSS
  $form['rss'] = array(
    '#type' => 'fieldset',
    '#title' => t('RSS settings'),
  );

  // RSS: Paths
  $form['rss']['subscribe_to_feed_rss_paths'] = array(
    '#type' => 'textarea',
    '#title' => t('Paths that are RSS feeds.'),
    '#default_value' => variable_get('subscribe_to_feed_rss_paths', '*/rss.xml
*/rss.xml?*
http://feeds.*'),
    '#description' => t("Enter one path per line. The '*' character is a wildcard."),
  );


  // Podcasts
  $form['podcasts'] = array(
    '#type' => 'fieldset',
    '#title' => t('Podcast settings'),
  );

  // Podcasts: Paths
  $form['podcasts']['subscribe_to_feed_podcast_paths'] = array(
    '#type' => 'textarea',
    '#title' => t('Paths that should only display podcast specific services.'),
    '#default_value' => variable_get('subscribe_to_feed_podcast_paths', '*podcast*
*video*'),
    '#description' => t("Enter one path per line. The '*' character is a wildcard."),
  );

  // Icons
  $form['icons'] = array(
    '#type' => 'fieldset',
    '#title' => t('Icon settings'),
  );
  $form['icons']['subscribe_to_feed_images_path'] = array(
    '#type' => 'textfield',
    '#title' => t("Alternative images directory"),
    '#description' => t('If you have alternative images directory for icons, enter the path from your index.php without a trailing slash. (ex: %path1 or %path2)', array('%path1' => 'files/newicons', '%path2' => 'sites/all/files/newicons')),
    '#default_value' => variable_get('subscribe_to_feed_images_path', ''),
  );

  $form['#validate'][] = 'subscribe_to_feed_admin_validate';

  return system_settings_form($form);
}

/**
 * Check if domain address textbox is empty
 */
function subscribe_to_feed_admin_validate($form, &$form_state) {
  if (!empty($form_state['values']['subscribe_to_feed_images_path']) && !file_exists($form_state['values']['subscribe_to_feed_images_path'])) {
    form_set_error('subscribe_to_feed_images_path', t("Alternative icon's folder not found. Please check the path and the folder's permissions."));
  }
}

////////////////////////////////////////////////////////////////////////////////
// Services form
////////////////////////////////////////////////////////////////////////////////

/**
 * Form builder; Services page for the 'Subscribe to feed' module.
 *
 * @ingroup forms
 */
function subscribe_to_feed_admin_services() {
  // Services: Code inspired by the menu_block.module admin settings form.
  $form['services'] = array(
    '#theme' => 'subscribe_to_feed_services_order',
    '#tree' => TRUE,
  );

  $services = subscribe_to_feed_get_services();

  $order = 0;
  // Append selected (checked) services first.
  $selected_services = variable_get('subscribe_to_feed_services', array());
  foreach ($selected_services as $service_name) {
    if (isset($services[$service_name])) {
      _subscribe_to_feed_admin_services_append_field_name($form['services'], $service_name, $order++, 1);
      unset($services[$service_name]);
    }
  }

  // Add remaining unassigned (unchecked) services.
  foreach ($services as $service_name => $service) {
    _subscribe_to_feed_admin_services_append_field_name($form['services'], $service_name, $order++, 0);
  }

  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Save configuration'),
  );

  return $form;
}


/**
 * Form submission handler; For subscribe to feed settings page.
 */
function subscribe_to_feed_admin_services_submit($form, &$form_state) {
  // Set subscribe_to_feed_services
  if (isset($form_state['values']['services'])) {
    $services = array();
    foreach ($form_state['values']['services'] AS $service_name => $row) {
      if ($row['show']) {
        // Add available field and its weight to list.
        $services[$service_name] = (int) $row['weight'];
      }
    }
    // Sort the keys by the weight stored in the value.
    asort($services);
    // Set field value to field name.
    foreach ($services AS $service_name => $weight) {
      $services[$service_name] = $service_name;
    }
    variable_set('subscribe_to_feed_services', $services);
  }

  // Display message
  drupal_set_message(t('The configuration options have been saved.'));
}

////////////////////////////////////////////////////////////////////////////////
// Sortable field helper and theme functions.
////////////////////////////////////////////////////////////////////////////////

/**
 * Append service to selected services orderable table.
 */
function _subscribe_to_feed_admin_services_append_field_name(&$element, $service_name, $order, $enabled) {
  $services = subscribe_to_feed_get_services();
  $total_services = count($services);
  $service = $services[$service_name];

  if ($service['url']) {
    $url = parse_url($service['url']);
    $site_url = 'http://'. $url['host'];
    $site_link = ' ('. l($site_url, $url['host']) .')';
  }
  else {
    $site_link = '';
  }

  $element[$service_name] = array(
    'title' => array(
      '#value' => subscribe_to_feed_get_image($service_name) .' '. check_plain($service['title']) . $site_link,
    ),
    'show' => array(
      '#type' => 'checkbox',
      '#default_value' => $enabled,
    ),
    'weight' => array(
      '#type' => 'weight',
      '#default_value' => $order - $total_services,
      '#delta' => $total_services,
      '#id' => 'edit-nodereference-backref-fields-'. $service_name,
    ),
  );
}

/**
 * Theme a drag-to-reorder table of field selection checkboxes.
 */
function theme_subscribe_to_feed_services_order($element) {
  // Set table header
  $header = array(
    t('Service name'),
    t('Show'),
    t('Weight'),
  );

  // Generate table of draggable field names.
  $rows = array();
  foreach (element_children($element) as $service_name) {
    $element[$service_name]['weight']['#attributes']['class'] = 'service-weight';
    $rows[] = array(
      'data' => array(
        drupal_render($element[$service_name]['title']),
        drupal_render($element[$service_name]['show']),
        drupal_render($element[$service_name]['weight']),
      ),
      'class' => 'draggable',
    );
  }

  // Generate unique id for tabledrag and theme the table.
  $id = form_clean_id('subscribe-to-feed-services');
  drupal_add_tabledrag($id, 'order', 'sibling', 'service-weight');
  return theme('table', $header, $rows, array('id' => $id));
}
